SystemOrganization addCategory: #'GT-Inspector'!
SystemOrganization addCategory: 'GT-Inspector-Examples'!

!TClassDescription methodsFor: '*GT-Inspector-Examples' stamp: 'TudorGirba 11/2/2014 19:51'!
examplesProvider

	^ self! !

!TClassDescription methodsFor: '*GT-Inspector-Examples' stamp: 'TudorGirba 11/2/2014 19:51'!
gtDisplayExamplesIn: aComposite

	^ aComposite tree
		title: 'E.g.';
		when: [ self gtExamples notEmpty ];
		display: [ self gtExamples ];
		children: [ :anExample | anExample childrenExamples ];
		format: [ :anExample | anExample title gtDisplayString ];
		send: [ :anExample | anExample result ];
		allExpanded;
		selectionPopulate: #selection 
			on: $o 
			entitled: 'Open source' 
			with: [ :list | list rawSelection methodReference ]! !

!TClassDescription methodsFor: '*GT-Inspector-Examples' stamp: 'TudorGirba 11/2/2014 19:50'!
gtExamples

	^	self gtExamplesExtractionStrategy rootExamplesFor: self! !

!TClassDescription methodsFor: '*GT-Inspector-Examples' stamp: 'TudorGirba 11/2/2014 19:50'!
gtExamplesExtractionStrategy

	^ GTExampleExtractionStrategy new! !

!TClassDescription methodsFor: '*GT-Inspector-Examples' stamp: 'AndreiChis 1/14/2015 15:35'!
gtInspectorExamplesIn: composite
	"This provides a list of example objects for the current class"
	<gtInspectorPresentationOrder: 20>
	<gtInspectorTag: #examples>
	|exampleProvider|

	exampleProvider := self examplesProvider.
	(exampleProvider gtDisplayExamplesIn: composite)
			"when: [ self exampleProvider rootExamples notEmpty ];"
			morphicSelectionAct: [ :list | 
					list rawSelection methodReference browse ] 
				icon: GLMUIThemeExtraIcons glamorousBrowse 
				on: $b 
				entitled: 'Browse source'
! !

GLMCompositePresentation subclass: #GTInspector
	instanceVariableNames: 'presentationFilter'
	classVariableNames: 'GTInspectorEnabledStatus GTNumberOfInspectorPanes'
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspector commentStamp: 'TudorGirba 8/21/2011 16:57' prior: 0!
This is meant to be used for inspecting objects. It offers multiple views and it uses a finder as a navigation. One particular feature is that you can use the evaluator tab to enter code, and evaluating it results in opening another pane to the right.

The object finder asks dynamically the object for the actual presentations that are displayed in each pane.

Example:
self openOn: Smalltalk.

Register it as a replacement for explorer:
self registerToolsOn: Smalltalk tools.!

!GTInspector class methodsFor: 'openning' stamp: 'TudorGirba 11/1/2013 22:16'!
explore: anObject 
	self openOn: anObject.
	^ anObject! !

!GTInspector class methodsFor: 'inspecting' stamp: 'TudorGirba 3/8/2014 07:51'!
gtInspectorExtensionsIn: composite
	<gtInspectorPresentationOrder: 40>
	composite list
		title: 'Extensions';
		display: [ self inspectorExtendingMethods ];
		showOnly: 50;
		format: [ :each | each gtDisplayString ];
		filterOn: [ :text :each | each gtDisplayString includesSubstring: text caseSensitive: false ]! !

!GTInspector class methodsFor: 'class initialization' stamp: 'TudorGirba 10/7/2012 22:50'!
initialize 
	self numberOfInspectorPanes: 2! !

!GTInspector class methodsFor: 'openning' stamp: 'TudorGirba 11/1/2013 22:16'!
inspect: anObject 
	self openOn: anObject.
	^ anObject! !

!GTInspector class methodsFor: 'openning' stamp: 'AndreiChis 4/16/2014 18:15'!
inspect: anObject label: aString
	self new 
		title: aString;
		openOn: anObject.
	^ anObject
! !

!GTInspector class methodsFor: 'openning' stamp: 'AndreiChis 10/28/2014 13:53'!
inspector: anObject

	^ self openOn: anObject! !

!GTInspector class methodsFor: 'inspecting' stamp: 'TudorGirba 2/7/2014 08:31'!
inspectorExtendingMethods
	^ (Object withAllSubclasses
		flatCollect: [ :each | 
			(Pragma allNamed: #gtInspectorPresentationOrder: in: each) 
				collect: [ :p | p method asRingDefinition ] ])
		sorted! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 8/29/2014 00:48'!
isGTInspectorEnabled

	^ GTInspectorEnabledStatus ifNil: [ GTInspectorEnabledStatus := Smalltalk tools inspector = self ]! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 10/7/2012 22:46'!
numberOfInspectorPanes
	^ GTNumberOfInspectorPanes ! !

!GTInspector class methodsFor: 'settings' stamp: 'TudorGirba 10/7/2012 22:46'!
numberOfInspectorPanes: anInteger
	GTNumberOfInspectorPanes := anInteger ! !

!GTInspector class methodsFor: 'openning' stamp: 'TudorGirba 11/25/2012 22:53'!
openOn: anObject 
	^ self new openOn: anObject! !

!GTInspector class methodsFor: 'tools registry' stamp: 'AndreiChis 8/28/2014 21:21'!
register

	self registerToolsOn: Smalltalk tools
! !

!GTInspector class methodsFor: 'tools registry' stamp: 'TudorGirba 11/1/2013 22:17'!
registerToolsOn: registry
	"self registerToolsOn: Smalltalk tools " 
	registry register: self as: #inspector
! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 8/28/2014 21:27'!
setGTInspectorEnabledStatus: aBoolean
	| oldStatus |
	
	oldStatus := self isGTInspectorEnabled.
	GTInspectorEnabledStatus := aBoolean.
	oldStatus ~= GTInspectorEnabledStatus ifTrue: [ 
		GTInspectorEnabledStatus 
			ifTrue: [ self register ]
			ifFalse: [ EyeInspector registerToolsOn: Smalltalk tools ] ]
	
	! !

!GTInspector class methodsFor: 'settings' stamp: 'AndreiChis 8/28/2014 21:46'!
settingsOn: aBuilder 
	<systemsettings>
	
	(aBuilder group: #gt)
		label: 'Glamorous Toolkit';
		with: [
			(aBuilder setting: #numberOfInspectorPanes)
				target: self; 
				label: 'Number of inspector panes';
		 		description: 'Specifies the maximum number of panes that are visible at every moment in a Glamorous Inspector.'.
			(aBuilder setting: #GTInspectorStatus)
				target: self;
				getSelector: #isGTInspectorEnabled;
				setSelector: #setGTInspectorEnabledStatus:; 
				label: 'GTInspector';
		 		description: 'Enable or disable the GTInspector.'.]! !

!GTInspector methodsFor: 'callbacks' stamp: 'AndreiChis 10/5/2014 21:27'!
actOnPaneAdded: ann

	self panes size > 1 ifTrue: [ 
		(self panes before: self panes last) updateToolbar "announce: (GLMPresentationToolbarUpdated new presentation: ann presentation)" ]! !

!GTInspector methodsFor: 'actions' stamp: 'EstebanLorenzano 12/7/2014 09:47'!
closeActionFor: aPresentation

	^ GLMGenericAction new
			action: [ self removeLastPane ];
			icon: GLMUIThemeExtraIcons glamorousClose;
			condition: [ :currentPresentation |
				self panes size > 1 and: [ self panes last presentations = currentPresentation ] ];
			title: 'Close' ! !

!GTInspector methodsFor: 'building' stamp: 'AndreiChis 12/13/2014 12:07'!
compose
	self title: [:anObject | 'Inspector on ', (self printObjectAsAnItem: anObject)].
	self act: [:b | b update] icon: GLMUIThemeExtraIcons glamorousRefresh entitled: 'Update'.
	"self act: [:b | b presentationFilter ifNotNil: [ :filter | filter changeFilterRequestFor: b] ] icon: GLMUIThemeExtraIcons glamorousSpawn entitled: 'Edit context'."
	self pager with: [ :browser | 
		browser fixedSizePanes: self class numberOfInspectorPanes.
		browser show: [ :a :each |
			a title: [ self printObjectAsAnItem: each ].
			a dynamicActions: [ :aPresentation | 
				each gtInspectorActions asOrderedCollection ].
			a titleAction: [ :aPresentation | self closeActionFor: aPresentation ].
			a dynamic 
				display: [ :x | 
					(GLMCompositePresentation new with: [:d | 
						each 
							gtInspectorPresentationsIn: d 
							inContext: self ])
							startOn: x ] ] ].
			
	self first when: GLMPaneAdded send: #actOnPaneAdded: to: self! !

!GTInspector methodsFor: 'accessing' stamp: 'TudorGirba 3/13/2014 13:39'!
entities
	"returns the list of objects from the #entity port of each pane of the finder"
	^ self first panes collect: [ :each | 
		(each port: #entity) value ]! !

!GTInspector methodsFor: 'initialization' stamp: 'AndreiChis 1/14/2015 15:43'!
initialize
	super initialize.
	
	self presentationsFilter: (GTInspectorTagFilterStrategy new
			defaultTag: #custom;
			addAndSelectTag: #basic;
			addAndSelectTag: #custom;
			addAndSelectTag: #examples)! !

!GTInspector methodsFor: 'actions' stamp: 'TudorGirba 8/21/2011 08:56'!
openExplorerFor: anObject
	"this method is needed for compatibility with ObjectExplorer"
	self openOn: anObject! !

!GTInspector methodsFor: 'accessing' stamp: 'TudorGirba 3/17/2014 16:30'!
panes
	^ self first panes! !

!GTInspector methodsFor: 'context' stamp: 'AndreiChis 12/9/2014 22:40'!
presentationFilter

	^ presentationFilter! !

!GTInspector methodsFor: 'context' stamp: 'AndreiChis 1/14/2015 15:53'!
presentationsFilter: aPresentationFilterStragery
	"Set a filter for limiting the number of visible presentations."
	presentationFilter := aPresentationFilterStragery! !

!GTInspector methodsFor: 'accessing' stamp: 'AndreiChis 8/21/2014 23:37'!
previousEntity
	^ self entities reverse second! !

!GTInspector methodsFor: 'accessing' stamp: 'AndreiChis 8/22/2014 01:29'!
previousEntityTo: anObject
	| entities index |
	entities := self entities.
	index := entities indexOf: anObject.
	^ index < 2 
		ifTrue: [ nil ]
		ifFalse: [ entities at: index - 1 ]! !

!GTInspector methodsFor: 'private' stamp: 'SvenVanCaekenberghe 10/6/2014 16:16'!
printObjectAsAnItem: anObject
	| header printString |
	header := (anObject class name first isVowel
		ifTrue: [ 'an ' ]
		ifFalse: [ 'a ' ]) , anObject class name.
	printString := (GTObjectPrinter new asTruncatedTextFrom: anObject) asString.
	^ (printString beginsWith: header)
		ifTrue: [ printString ]
		ifFalse: [ 
			String streamContents: [ :out | 
				out << header; << ' ('; << printString; << ')' ] ]! !

!GTInspector methodsFor: 'actions' stamp: 'AndreiChis 10/5/2014 21:28'!
removeLastPane
	| lastPane |
	
	lastPane := self first panes last.
	self first removeLastPane.
	self first announce: (GLMPaneRemoved pane: lastPane fromBrowser: self first).
	self first panes last updateToolbar! !

!GTInspector methodsFor: 'context' stamp: 'AndreiChis 1/14/2015 14:20'!
shouldDisplayPresentationCreateBy: aMethod
	"Verify whether or not the presentation created by the given method should be displayed.
	If no filter is given display all presentations."
	^ self presentationFilter 
		ifNil: [ true]
		ifNotNil: [ self presentationFilter shouldDisplayPresentationCreateBy: aMethod ]! !

!GTInspector methodsFor: 'scripting' stamp: 'AndreiChis 8/18/2014 19:32'!
showFirst: aBlock

	self first showFirst: aBlock! !

!GTInspector methodsFor: 'actions' stamp: 'AndreiChis 10/29/2014 14:43'!
updatePanesFor: anObject

	(self panes select: [ :each | 
		(each port: #entity) value == anObject ]) do: [ :each | each update ]! !

GLMCompositePresentation subclass: #GTObjectVariablesBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTObjectVariablesBrowser commentStamp: 'TudorGirba 8/22/2011 12:52' prior: 0!
This is a helper browser for the main view in the GTInspector.

Example:
self openOn: Smalltalk!

!GTObjectVariablesBrowser methodsFor: 'building' stamp: 'TudorGirba 10/22/2014 14:39'!
compose
	self title: 'Raw'.
	self tabulator with: [ :browser |
		browser row: #variables; row: #evaluator.
		browser transmit to: #variables; andShow: [:a | 
			self variablesIn: a ].
		browser transmit to: #evaluator; andShow: [:a | 
			self evaluatorIn: a ].
		browser transmit from: #variables; toOutsidePort: #selection; transformed: [:assoc | assoc ifNotNil: [assoc value]].
		browser transmit from: #evaluator; toOutsidePort: #selection ]! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'TudorGirba 10/17/2014 15:28'!
evaluatorIn: composite
	^ composite pharoPlayground
		doItReceiver: #yourself;
		display: [ :object | 
			String streamContents: [ :stream | 
				stream 
					nextPutAll: (self printStringOf: object);
					cr;
					nextPutAll: (GTSnippets snippetAt: object class)] ];
		onChangeOfPort: #text act: [ :text :object |
				GTSnippets 
					snippetAt: object class 
					put: (self stringWithoutInitialCommentFrom: text text asString) ];
		installGoAction;
		installDefaultSelectionActions! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 8/4/2014 21:26'!
iconFor: anObject
		
	^ [ (anObject iconOrThumbnailOfSize: 16)
		ifNil: [ anObject class systemIcon ] ]
			on: Error do: [ :error |
			self iconFor: anObject error: error ]! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 8/4/2014 21:25'!
iconFor: anEyeElement error: error
	^ Smalltalk ui icons smallWarningIcon! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 8/28/2014 19:52'!
printStringOf: anObject
	^ [(anObject printString contractTo: 200) asComment] 
			on: Error 
			do: ['"Error printing. Try self printString to debug"']! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 1/9/2015 20:58'!
stringWithoutInitialCommentFrom: aString
	^ (aString trimLeft isEmpty not and: [ aString trimLeft first = $" ])
		ifFalse: [ aString trimBoth ]
		ifTrue: [|startIndex|
			startIndex := aString findAnySubStr: '"' startingAt: (aString indexOf: $") + 1.
			(startIndex = (aString size + 1)) ifTrue: [ "No content if a comment cannot be identified. " ^ '' ].
			(aString
				copyFrom: startIndex + 1 
				to: aString size) trimBoth ]! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 10/10/2014 18:36'!
updateInstanceVariable: aSymbol from: anObject basedOn: aString
	|result|

	(anObject class hasInstVarNamed: aSymbol) 
		ifFalse: [ ^ self ].
	result := anObject class compiler
			source: aString;
			context: nil;
			receiver: anObject;
			failBlock: [ ^ self ];
			evaluate.
						
	anObject instVarNamed: aSymbol put: result.
	! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'AndreiChis 8/5/2014 14:19'!
variableValuePairsFor: anObject
	^ anObject gtInspectorVariableValuePairs asSortedCollection asOrderedCollection 
		yourself! !

!GTObjectVariablesBrowser methodsFor: 'private building' stamp: 'TudorGirba 12/5/2014 13:53'!
variablesIn: composite
	^ composite table
		showOnly: 50;
		shouldValidate: false;
		children: [ :assoc | assoc key = 'self'
			ifTrue: [ #() ]
			ifFalse: [ self variableValuePairsFor: assoc value ] ];
		icon: [ :assoc | self iconFor: assoc value ];
		display: [ :anObject | (self variableValuePairsFor: anObject)
				addFirst: 'self' -> anObject;
				yourself ];
		column: 'Variable' 
			evaluated: [:assoc | '  ',(GTObjectPrinter new asNonTruncatedTextFrom: assoc key) ];
		column: 'Value' 
			evaluated: [:assoc | GTObjectPrinter new asTruncatedTextFrom: assoc value ]
			modified: [:newValue :assoc :presentation | 
				self updateInstanceVariable: assoc key from: presentation entity basedOn: newValue.
				presentation update ];
		"send: #value;"
		morphicSelectionAct: [:list | list selection value browse ] 
			icon: GLMUIThemeExtraIcons glamorousBrowse 
			on: $b 
			entitled: 'Browse';
		morphicSelectionAct: [:list | list selection value inspect ] 
			icon: GLMUIThemeExtraIcons glamorousInspect 
			on: $i 
			entitled: 'Inspect';
		selectionAct: [:list | 
			((list selection pointersToExcept: { list selection })
									reject: [ :each | each pointsOnlyWeaklyTo: list selection ]) inspect ] 
			on: $t
			entitled: 'Open pointers to'! !

GLMCompositePresentation subclass: #GTSimpleMethodsBrowser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTSimpleMethodsBrowser commentStamp: 'TudorGirba 1/18/2014 14:29' prior: 0!
This browser is used to browse the class structure of every object in the inspector.

Example:
self openOn: World!

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 11/24/2013 08:37'!
accumulatedSourceOf: m for: object in: composite
	composite accumulator with: [ :acc | 
		acc show: [ :a |
			a title: [ :method | method selector ].
			self sourceOf: m for: object in: a ] ]! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 12/21/2013 23:05'!
classesIn: a
	^ a list
		display: [ :object | 
			| all |
			all := OrderedCollection new.
			object class withAllSuperclassesDo: [:each | all add: each ].
			all ];
"		initialize: [ :list | list selection: list entity class ];"
		format: [ :class | class name ];
		morphicSelectionAct: [ :list | Smalltalk tools browser fullOnClass: list selection ]
			icon: GLMUIThemeExtraIcons glamorousBrowse
			on: $b
			entitled: 'Browse'! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 3/9/2014 08:59'!
compose
	self title: 'Meta'.
	self tabulator with: [ :browser |
		browser
			row: [ :r | r column: #classes; column: #methods span: 3 ];
			row: #source.
		browser transmit
			to: #classes;
			andShow: [ :a | self classesIn: a ].
		browser transmit
			to: #classes port: #selection;
			transformed: [:each | each class ].
		browser transmit
			fromOutsidePort: #entity;
			from: #classes;
			to: #methods;
			andShow: [ :a | self methodsIn: a ].
		browser transmit
			from: #methods;
			fromOutsidePort: #entity;
			to: #source;
			when: [ :method | method notNil ];
			andShow: [ :a :method :object | self sourceOf: method for: object in: a ].
		browser transmit
			toOutsidePort: #selection;
			from: #methods port: #evaluatedObject.
		browser transmit
			toOutsidePort: #selection;
			from: #source port: #evaluatedObject.
	]! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 10/3/2014 15:14'!
evaluationAction
	^ GLMGenericAction new
		action: [ :listPresentation :obj | 
					| result |
					result := obj perform: listPresentation selection selector.
					(listPresentation pane port: #evaluatedObject) value: result ];
		shortcut: $g;
		title: 'Do it and go';
		yourself! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 7/15/2012 18:58'!
methodActionsFor: aMethod
	^ aMethod selector isUnary
		ifTrue: [ Array with: self evaluationAction ]
		ifFalse: [ #() ]! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 12/17/2013 08:00'!
methodsIn: a
	^ a list
		display: [ :object :class | class methods collect: #asRingDefinition];
		filterOn: [ :text :each | (text , '*') match: each selector ];
		tags: [ :method | method category ];
		sorted: [ :method1 :method2 | method1 selector < method2 selector ];
		format: [ :method | method selector ];
		morphicSelectionAct: [ :list :object :class | 
			Smalltalk tools browser 
				fullOnClass: class 
				selector: list selection selector ]
			icon: GLMUIThemeExtraIcons glamorousBrowse
			on: $b
			entitled: 'Browse';
		dynamicActionsOnSelection: [ :list | self methodActionsFor: list selection ]! !

!GTSimpleMethodsBrowser methodsFor: 'building' stamp: 'TudorGirba 11/2/2014 22:04'!
sourceOf: method for: object in: composite
	^ composite smalltalkCode
		display: [ :m | m sourceCode ];
		smalltalkClass: [ :m | m methodClass ];
		doItReceiver: [ object ];
		selectionPopulate: #evaluatedObject
			on: $g
			entitled: 'Do it and go'
			with: [ :text | Smalltalk compiler evaluate: text selectedText for: object logged: false ];
		act: [ :text :m | self updateOuterPaneFrom: text for: method ]
			icon: GLMUIThemeExtraIcons glamorousAccept
			on: $s
			entitled: 'Accept';
		act: [ :text :m | 
			Smalltalk tools browser 
				fullOnClass: m methodClass 
				selector: m selector ]
			icon: GLMUIThemeExtraIcons glamorousBrowse
			entitled: 'Browse';
		act: [ :text :m | 
			(text pane port: #evaluatedObject) value: m ]
			icon: GLMUIThemeExtraIcons glamorousInspect
			entitled: 'Inspect'! !

!GTSimpleMethodsBrowser methodsFor: 'private' stamp: 'TudorGirba 12/17/2013 15:27'!
updateOuterPaneFrom: text for: method
	"this is a utility method to tell the pane surrounding the browser to update.
	The reason we do that is to support the use case of updating the GTInspector from
	the editor by modifying code directly in the Inspector."
	
	| outerPane newMethod |
	newMethod := method methodClass compile: text text notifying: nil.
	outerPane := text pane browser pane.
	outerPane update.
	self
		flag:
			'This is a huge hack because of so many reasons:
				- the presentation accesses the state of the outer browser directly
				- the presentation depends on the implementation of the browser 
				It should be solved through transmissions.
				
				But, it works for now and the browser is reasonably updated'.
	^ outerPane
		withAllPanesAndPresentationsDo: [ :child | 
			(child isKindOf: self class)
				ifTrue: [ 
					((child first paneNamed: #classes) port: #selection) value: method methodClass.
					((child first paneNamed: #methods) port: #selection)
						value:
							((child first paneNamed: #methods) presentations first displayedValue detect: [ :each | each selector = newMethod ]) ] ]! !

Object subclass: #GTExample
	instanceVariableNames: 'result title provider selector parentExample extractionStrategy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector-Examples'!

!GTExample commentStamp: 'AndreiChis 10/30/2014 21:35' prior: 0!
This encapsulates one example.!

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 15:39'!
childrenExamples
	
	^ self extractionStrategy 
		ifNil: [ OrderedCollection new ]
		ifNotNil: [ self extractionStrategy childrenExamplesFor: self ]! !

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 15:39'!
extractionStrategy
	^ extractionStrategy! !

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 15:39'!
extractionStrategy: anObject
	extractionStrategy := anObject! !

!GTExample methodsFor: 'presentations' stamp: 'AndreiChis 10/30/2014 14:35'!
gtDisplaySourceIn: composite

	^ self methodReference gtInspectorSourceIn: composite! !

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 15:07'!
hasParentExample

	^ parentExample notNil! !

!GTExample methodsFor: 'testing' stamp: 'AndreiChis 10/30/2014 14:24'!
isGTExample
	
	^ true! !

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 12:26'!
methodReference

	^ (self provider class compiledMethodAt: self selector) asRingDefinition! !

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 15:07'!
parentExample: anExample

	parentExample := anExample! !

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 12:23'!
provider

	^ provider! !

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 12:23'!
provider: anObject

	provider := anObject! !

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 21:45'!
result

	^ result ifNil: [ result := self hasParentExample
			ifFalse: [ self provider perform: self selector ]
			ifTrue: [ self provider perform: self selector with: parentExample result ] ]! !

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 12:23'!
selector

	^ selector! !

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 12:23'!
selector: aSymbol

	selector := aSymbol! !

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 12:20'!
title

	^ title 
		ifNil: [ self result ]
		ifNotNil: [ title ]! !

!GTExample methodsFor: 'accessing' stamp: 'AndreiChis 10/30/2014 12:20'!
title: aString

	title := aString! !

Object subclass: #GTExampleExtractionStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector-Examples'!

!GTExampleExtractionStrategy commentStamp: 'AndreiChis 10/30/2014 21:35' prior: 0!
I extract and create examples from an example provider.!

!GTExampleExtractionStrategy methodsFor: 'example creation' stamp: 'AndreiChis 10/31/2014 11:00'!
childrenExamplesFor: anExample

	^ ((Pragma 
		allNamed: self exampleDependencyPragma 
		in: anExample provider class) select: [ :aPragma | 
			(aPragma argumentAt: 1) = anExample selector]) 
				ifEmpty: [ #() ]
				ifNotEmpty: [ :pragmas | 
					pragmas collect: [ :each | 
						(self createExampleFromDependencyPragma: each from: anExample provider) ] ]! !

!GTExampleExtractionStrategy methodsFor: 'example creation' stamp: 'AndreiChis 10/31/2014 10:53'!
createExampleForExamplePragma: aPragma from: anExampleProvider
	| example |
	
	example := self createExampleForMethod: aPragma method asRingDefinition from: anExampleProvider.
	aPragma arguments size = 1 ifTrue: [ 
		example title: (aPragma argumentAt: 1) ].
	^ example! !

!GTExampleExtractionStrategy methodsFor: 'example creation' stamp: 'AndreiChis 10/31/2014 10:53'!
createExampleForMethod: aMethod from: anExampleProvider
	| newExample |

	newExample := self exampleClass new
		selector: aMethod selector;
		extractionStrategy: self;
		provider: anExampleProvider.
	newExample parentExample: (self parentExampleFor: newExample).
	^ newExample! !

!GTExampleExtractionStrategy methodsFor: 'example creation' stamp: 'AndreiChis 10/31/2014 10:54'!
createExampleFromDependencyPragma: aPragma from: anExampleProvider

	^ self 
		createExampleForExamplePragma: (aPragma method pragmas detect: [:each | 
				self examplePragmas includes: each keyword])
		from: anExampleProvider! !

!GTExampleExtractionStrategy methodsFor: 'accessing' stamp: 'AndreiChis 10/31/2014 10:48'!
exampleClass

	^ GTExample! !

!GTExampleExtractionStrategy methodsFor: 'accessing' stamp: 'AndreiChis 10/31/2014 11:00'!
exampleDependencyPragma

	^ #exampleFrom:! !

!GTExampleExtractionStrategy methodsFor: 'accessing' stamp: 'AndreiChis 10/31/2014 10:59'!
examplePragmas

	^ #(example example:)! !

!GTExampleExtractionStrategy methodsFor: 'example creation' stamp: 'AndreiChis 10/31/2014 11:00'!
parentExampleFor: anExample
	| dependencyPragma |
	
	dependencyPragma := anExample methodReference compiledMethod pragmas 
		detect: [ :each | each keyword = self exampleDependencyPragma ] 
		ifNone: [ nil ].

	^ dependencyPragma 
		ifNil: [ nil ]
		ifNotNil: [
			self 
				createExampleForMethod: (anExample methodReference realClass 
					compiledMethodAt: (dependencyPragma argumentAt: 1))
				from: anExample provider ].! !

!GTExampleExtractionStrategy methodsFor: 'example creation' stamp: 'AndreiChis 10/31/2014 10:59'!
rootExamplesFor: anExampleProvider
	| pragmas |

	pragmas := self examplePragmas 
		inject: OrderedCollection new
		into: [ :result :pragma |
			result , (Pragma 
				allNamed: pragma 
				in: anExampleProvider class)].
	^ (pragmas select: [ :each | each selector isUnary ])
			ifEmpty: [ #() ]
			ifNotEmpty: [ :examplePragmas | examplePragmas collect: [ :eachPragma | 
				self createExampleForExamplePragma: eachPragma from: anExampleProvider ] ]! !

Object subclass: #GTInspectorPresentationFilterStrategy
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorPresentationFilterStrategy commentStamp: 'AndreiChis 1/14/2015 14:19' prior: 0!
A GTInspectorPresentationFilterStrategy is the root class for filters that can limit the number if visible presentations for an object.

Subclasses need to implement  #shouldDisplayPresentationCreateBy: to indicate whether or not a give presentation should be displayed.
!

GTInspectorPresentationFilterStrategy subclass: #GTInspectorMethodListFilterStrategy
	instanceVariableNames: 'allMethodSignatures selectedMethodSignatures'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorMethodListFilterStrategy commentStamp: 'AndreiChis 1/14/2015 15:52' prior: 0!
GTInspectorMethodListFilterStrategy implements a filtering strategy for presentations based on method signatures (method selector and class).

The filter returns true for a method if the signature of that method is selected in the filter. To be taken into account during filtering a signature must be first added to the filter and the selected. Only signatures that have been first added can be selected. !

!GTInspectorMethodListFilterStrategy class methodsFor: 'examples' stamp: 'AndreiChis 1/14/2015 15:45'!
exampleASTBrowser
	"An inspector showing a browser for navigating through AST nodes."
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationsFilter: (GTInspectorMethodListFilterStrategy new
		addAndSelectSignatureFor: CompiledMethod>>#gtInspectorASTIn:;
		addAndSelectSignatureFor: RBProgramNode>>#gtInspectorSourceCodeIn:).
	inspector openOn: Morph>>#initialize.! !

!GTInspectorMethodListFilterStrategy class methodsFor: 'examples' stamp: 'AndreiChis 1/14/2015 15:43'!
exampleCodeBrowser
	"An inspector showing only the Meta presentation for all objects"
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationsFilter: (GTInspectorMethodListFilterStrategy new
		addAndSelectSignatureFor: Object>>#gtInspectorMetaIn:).
	inspector openOn: Morph new.! !

!GTInspectorMethodListFilterStrategy class methodsFor: 'examples' stamp: 'AndreiChis 1/14/2015 15:49'!
exampleFileBrowser
	"An inspector for browsing directories and files."
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationsFilter: (GTInspectorMethodListFilterStrategy new
		addAndSelectSignatureFor: FileReference>>#gtInspectorStScriptIn:;
		addAndSelectSignatureFor: FileReference>>#gtInspectorPngIn:;
		addAndSelectSignatureFor: FileReference>>#gtInspectorJpegIn:;
		addAndSelectSignatureFor: FileReference>>#gtInspectorItemsIn:;
		addAndSelectSignatureFor: FileReference>>#gtInspectorGifIn:;
		addAndSelectSignatureFor: FileReference>>#gtInspectorContentsIn:;
		addAndSelectSignatureFor: FileReference>>#gtInspectorFuelIn:).
	inspector openOn: FileSystem workingDirectory.! !

!GTInspectorMethodListFilterStrategy class methodsFor: 'examples' stamp: 'AndreiChis 1/14/2015 15:45'!
exampleFullInspector
	"An inspector showing all presentations for all objects"
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationsFilter: (GTInspectorMethodListFilterStrategy new
		addAndSelectAllSignatures).
	inspector openOn: Morph new.! !

!GTInspectorMethodListFilterStrategy class methodsFor: 'examples' stamp: 'AndreiChis 1/14/2015 15:43'!
exampleRawInspector
	"An inspector showing only the Raw presentation for all objects"
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationsFilter: (GTInspectorMethodListFilterStrategy new
		addAndSelectSignatureFor: Object>>#gtInspectorRawIn:).
	inspector openOn: Morph new.! !

!GTInspectorMethodListFilterStrategy methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:28'!
addAllSignatures
	
	self allPresentationMethods do: [ :aMethod | self addSignatureFor: aMethod ] ! !

!GTInspectorMethodListFilterStrategy methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:33'!
addAndSelectAllSignatures
	
	self allPresentationMethods do: [ :aMethod | self addAndSelectSignatureFor: aMethod ] ! !

!GTInspectorMethodListFilterStrategy methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:46'!
addAndSelectSignatureFor: aMethod
	|signature|
	
	signature := self signatureFor: aMethod.
	(allMethodSignatures includes: signature) ifFalse: [ allMethodSignatures add: signature ].
	(selectedMethodSignatures includes: signature) ifFalse: [ selectedMethodSignatures add: signature ].
	! !

!GTInspectorMethodListFilterStrategy methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:24'!
addSignatureFor: aMethod
	|signature|
	
	signature := self signatureFor: aMethod.
	(allMethodSignatures includes: signature) ifFalse: [ allMethodSignatures add: signature ].! !

!GTInspectorMethodListFilterStrategy methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 11:32'!
allPresentationMethods

	^ (PragmaCollector allSystemPragmas select: [ :pragma | 
		pragma key == self inspectorPragmaKey ]) collect: [ :aPragma | aPragma method ]! !

!GTInspectorMethodListFilterStrategy methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 12:06'!
changeFilterRequestFor: aGTInspector

	self
		openListChooseLabeled: 'All available presentations' 
		forInspector: aGTInspector
		with: allMethodSignatures 
		select: selectedMethodSignatures 
		format: [ :aSignature | self printSignature: aSignature ]
		actionSelector: #replaceSelectedSignaturesWith:in:! !

!GTInspectorMethodListFilterStrategy methodsFor: 'initialization' stamp: 'AndreiChis 12/13/2014 11:43'!
initialize
	super initialize.
	allMethodSignatures := OrderedCollection new.
	selectedMethodSignatures := OrderedCollection new.! !

!GTInspectorMethodListFilterStrategy methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 11:31'!
inspectorPragmaKey

	^ #gtInspectorPresentationOrder:! !

!GTInspectorMethodListFilterStrategy methodsFor: 'printing' stamp: 'AndreiChis 12/13/2014 12:06'!
printSignature: aSignature

	^ aSignature first name, '>>', aSignature second asString! !

!GTInspectorMethodListFilterStrategy methodsFor: 'callbacks' stamp: 'AndreiChis 12/13/2014 11:46'!
replaceSelectedSignaturesWith: aList in: aGTInspector

	selectedMethodSignatures := aList.
	aGTInspector update! !

!GTInspectorMethodListFilterStrategy methodsFor: 'testing' stamp: 'AndreiChis 12/13/2014 12:03'!
shouldDisplayPresentationCreateBy: aMethod

	^ selectedMethodSignatures anySatisfy: [ :aSignature | 
		aSignature first = aMethod methodClass and: [ aSignature second = aMethod selector ] ]! !

!GTInspectorMethodListFilterStrategy methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 12:03'!
signatureFor: aMethod

	^ {aMethod methodClass . aMethod selector}! !

!GTInspectorPresentationFilterStrategy methodsFor: 'accessing' stamp: 'AndreiChis 12/9/2014 22:38'!
changeFilterRequestFor: aGTInspector

	self subclassResponsibility! !

!GTInspectorPresentationFilterStrategy methodsFor: 'private' stamp: 'AndreiChis 12/13/2014 12:04'!
openListChooseLabeled: sString forInspector: aGTInspector with: allElements select: selectedElements format: aBlockOrSelector actionSelector: aSymbol
	|treeModel|
	
	treeModel := MorphTreeModel new.
	treeModel
		 beCheckList;
		 rootItems: allElements;
		 wrapBlockOrSelector: aBlockOrSelector;
		 autoMultiSelection: true.
	 (treeModel openDialogWindowIn: World title: sString selectedtems: selectedElements) ifNotNil: [ :selection | 
		self perform: aSymbol with: selection with: aGTInspector ]! !

!GTInspectorPresentationFilterStrategy methodsFor: 'testing' stamp: 'AndreiChis 12/9/2014 21:27'!
shouldDisplayPresentationCreateBy: aMethod

	self subclassResponsibility! !

GTInspectorPresentationFilterStrategy subclass: #GTInspectorTagFilterStrategy
	instanceVariableNames: 'allTags selectedTags defaultTag'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTInspectorTagFilterStrategy commentStamp: 'AndreiChis 1/14/2015 15:51' prior: 0!
GTInspectorTagFilterStrategy implements a filtering strategy for presentations based on tags.

A tag is defined as the parameter of the prama <gtInspectorTag:>. A method can have one or more tags.
The filter returns true for a method if that method contains at least a tag selected in the filter. To be taken into account during filtering a tag must be first added to the filter and the selected. Only tags that have been first added can be selected.  

If a method does not define any tags the filter threats that method as having a single tag stored in the #defaultTag instance variable. The default defaultTag is #custom. The #basic tag  groups the Raw and Meta presentations. Inspect 'GTInspectorTagFilterStrategy new allPragmasWithTag' to view all pragmas creating the tags.!

!GTInspectorTagFilterStrategy class methodsFor: 'examples' stamp: 'AndreiChis 1/14/2015 14:44'!
exampleBasicInspector
	"An inspector showing only the basic presentations Raw and Meta for all object."
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationsFilter: (GTInspectorTagFilterStrategy new
		addAndSelectTag: #basic).
	inspector openOn: Morph new.! !

!GTInspectorTagFilterStrategy class methodsFor: 'examples' stamp: 'AndreiChis 1/14/2015 14:44'!
exampleCustomInspector
	"An inspector showing only the custom presentations for all object (excluding Raw and Meta)."
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationsFilter: (GTInspectorTagFilterStrategy new
		defaultTag: #custom;
		addAndSelectTag: #custom).
	inspector openOn: Morph new.! !

!GTInspectorTagFilterStrategy class methodsFor: 'examples' stamp: 'AndreiChis 1/14/2015 15:46'!
exampleExampleBrowser
	"A browser for viewing examples"
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationsFilter: (GTInspectorTagFilterStrategy new
		addAndSelectTag: #examples).
	inspector openOn: Dictionary.! !

!GTInspectorTagFilterStrategy class methodsFor: 'examples' stamp: 'AndreiChis 1/14/2015 15:21'!
exampleFullInspector
	"An inspector showing all the available presentations for all objects."
	|inspector|
	
	inspector := GTInspector new.
	inspector presentationsFilter: (GTInspectorTagFilterStrategy new
		defaultTag: #custom;
		addAndSelectTag: #custom;
		addAndSelectAllTags).
	inspector openOn: Morph new.! !

!GTInspectorTagFilterStrategy methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:15'!
addAllTags
	
	self allTags do: [ :aTag | self addTag: allTags ] ! !

!GTInspectorTagFilterStrategy methodsFor: 'adding' stamp: 'AndreiChis 1/14/2015 15:21'!
addAndSelectAllTags
	
	self allTags do: [ :aTag | self addAndSelectTag: aTag ] ! !

!GTInspectorTagFilterStrategy methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:16'!
addAndSelectTag: aSymbol

	self addTag: aSymbol.
	(selectedTags includes: aSymbol) ifFalse:  [selectedTags add: aSymbol]! !

!GTInspectorTagFilterStrategy methodsFor: 'adding' stamp: 'AndreiChis 12/13/2014 11:16'!
addTag: aSymbol

	(allTags includes: aSymbol) ifFalse:  [allTags add: aSymbol].! !

!GTInspectorTagFilterStrategy methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 11:13'!
allPragmasWithTag

	^ PragmaCollector allSystemPragmas select: [ :pragma | pragma key == self tagPragmaKey ]! !

!GTInspectorTagFilterStrategy methodsFor: 'accessing' stamp: 'AndreiChis 1/14/2015 15:18'!
allTags

	^ (self allPragmasWithTag collect: [ :pragma | 
		pragma argumentAt: 1 ]) asSet sorted: [ :a :b | a < b ]! !

!GTInspectorTagFilterStrategy methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 12:05'!
changeFilterRequestFor: aGTInspector

	self
		openListChooseLabeled: 'All available tags' 
		forInspector: aGTInspector
		with: allTags 
		select: selectedTags 
		format: #printString
		actionSelector: #replaceSelectedTagsWith:in:
! !

!GTInspectorTagFilterStrategy methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 00:38'!
defaultTag: aSymbol

	defaultTag := aSymbol! !

!GTInspectorTagFilterStrategy methodsFor: 'initialization' stamp: 'AndreiChis 12/13/2014 10:59'!
initialize
	super initialize.
	
	allTags := OrderedCollection new.
	selectedTags := OrderedCollection new.
	defaultTag := #custom! !

!GTInspectorTagFilterStrategy methodsFor: 'callbacks' stamp: 'AndreiChis 12/13/2014 11:40'!
replaceSelectedTagsWith: aListOfSymbols in: aGTInspector

	selectedTags := aListOfSymbols.
	aGTInspector update.! !

!GTInspectorTagFilterStrategy methodsFor: 'testing' stamp: 'AndreiChis 12/13/2014 11:03'!
shouldDisplayPresentationCreateBy: aMethod

	^ (self tagsFrom: aMethod) includesAny: selectedTags! !

!GTInspectorTagFilterStrategy methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 11:11'!
tagPragmaKey

	^ #gtInspectorTag:! !

!GTInspectorTagFilterStrategy methodsFor: 'accessing' stamp: 'AndreiChis 12/13/2014 11:11'!
tagsFrom: aMethod

	^ (aMethod pragmas select: [ :each | each key == self tagPragmaKey ])
		ifEmpty: [ defaultTag ifNil: [ {} ] ifNotNil: [ {defaultTag} ] ]
		ifNotEmpty: [ :currentPragmas | currentPragmas collect: [ :each | each argumentAt: 1 ] ]! !

Object subclass: #GTObjectPrinter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

!GTObjectPrinter commentStamp: 'TudorGirba 1/18/2014 14:29' prior: 0!
This is a utility class for printing objects to Text or String.!

!GTObjectPrinter methodsFor: 'printing' stamp: 'TudorGirba 12/23/2013 09:17'!
asNonTruncatedTextFrom: anObject
	^ [ Text fromString: (anObject asString replaceAll: String cr with: String space) ] 
			on: Error 
			do: [ Text string: 'error printing' attribute: TextColor red ]! !

!GTObjectPrinter methodsFor: 'printing' stamp: 'TudorGirba 9/6/2014 19:03'!
asTruncatedTextFrom: anObject
	^ [ Text fromString: (anObject gtDisplayString replaceAll: String cr with: String space) ]
		on: Error
		do: [ Text string: 'error printing' attribute: TextColor red ]! !

Object subclass: #GTSnippets
	instanceVariableNames: 'instance snippets'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'GT-Inspector'!

GTSnippets class
	instanceVariableNames: 'instance'!

!GTSnippets commentStamp: '<historical>' prior: 0!
GTSnippets instance!

GTSnippets class
	instanceVariableNames: 'instance'!

!GTSnippets class methodsFor: 'accessing' stamp: 'TudorGirba 11/27/2013 14:55'!
instance
	^ instance ifNil: [ instance := self new ]! !

!GTSnippets class methodsFor: 'accessing' stamp: 'TudorGirba 2/2/2014 14:05'!
reset
	instance := nil! !

!GTSnippets class methodsFor: 'accessing' stamp: 'TudorGirba 11/27/2013 15:00'!
snippetAt: anObject
	^ self instance snippetAt: anObject! !

!GTSnippets class methodsFor: 'accessing' stamp: 'TudorGirba 11/27/2013 15:10'!
snippetAt: anObject put: aString
	^ self instance snippetAt: anObject put: aString! !

!GTSnippets methodsFor: 'accessing' stamp: 'TudorGirba 12/30/2013 23:44'!
snippetAt: anObject
	^ self snippets 
		at: anObject 
		ifAbsent: [ 'self' ]! !

!GTSnippets methodsFor: 'accessing' stamp: 'TudorGirba 12/30/2013 23:44'!
snippetAt: anObject put: aString
	^ self snippets 
		at: anObject 
		put: aString! !

!GTSnippets methodsFor: 'accessing' stamp: 'TudorGirba 5/26/2014 12:39'!
snippetAtOrEmpty: anObject
	^ self snippets 
		at: anObject 
		ifAbsent: [ '' ]! !

!GTSnippets methodsFor: 'accessing' stamp: 'TudorGirba 11/27/2013 14:59'!
snippets 
	^ snippets ifNil: [ snippets := IdentityDictionary new ]! !

!Object methodsFor: '*GT-Inspector' stamp: 'TudorGirba 12/23/2013 09:12'!
gtDisplayOn: stream
	"This offers a means to customize how the object is shown in the inspector"
	^ self printOn: stream! !

!Object methodsFor: '*GT-Inspector' stamp: 'TudorGirba 12/28/2013 21:59'!
gtDisplayString
	"This offers a means to customize how the object is shown in the inspector.
	Do not override this method. Override gtDisplayOn: instead"
		
	| limitedString limit |
	limit := 1000.
	limitedString := String streamContents: [:s | self gtDisplayOn: s] limitedTo: limit.
	limitedString size < limit ifTrue: [^ limitedString].
	^ limitedString , ' ...'! !

!Object methodsFor: '*GT-Inspector' stamp: 'TudorGirba 2/27/2014 21:42'!
gtInspectorActionBrowse
	<gtInspectorAction>
	^ GLMGenericAction new
		action: [ self browse ];
		icon: GLMUIThemeExtraIcons glamorousBrowse;
		title: 'Browse'! !

!Object methodsFor: '*GT-Inspector' stamp: 'TudorGirba 2/27/2014 10:36'!
gtInspectorActions
	"This is a utility method that allows each object to define a set of actions 
	specific for itself. These actions are used when the object appears in the inspector.

	The actions are collected based on the #gtInspectorAction pragma that are defined in
	unary methods corresponding to the body of the actions"
	
	| all |
	all := (Pragma 
				allNamed: #gtInspectorAction
				from: self class 
				to: Object) collect: [:eachPragma |
					self perform: eachPragma selector ].
	^ all asSortedCollection: [:a :b | 
		| first second |
		first := a isCategorized ifTrue: [a category, a title] ifFalse: [a title].
		second := b isCategorized ifTrue: [b category, b title] ifFalse: [b title].
		first < second]! !

!Object methodsFor: '*GT-Inspector' stamp: 'AndreiChis 1/14/2015 15:34'!
gtInspectorExampleSourceIn: composite inContext: aGTInspector
	<gtInspectorPresentationOrder: 900>
	<gtInspectorTag: #examples>
	
	| previousRawSelection |
	(aGTInspector panes size > 1) 
		ifFalse: [ ^ self ].	
	previousRawSelection := (aGTInspector panes reversed second port: #rawSelection) value.
	((previousRawSelection isGTExample) "and: [
		aGTInspector first lastActivePresentation pane ~= aGTInspector panes reversed second ]")
			ifTrue: [ 
				(previousRawSelection gtDisplaySourceIn: composite) 
					title: 'E.g. source' ]! !

!Object methodsFor: '*GT-Inspector' stamp: 'TudorGirba 6/28/2014 22:18'!
gtInspectorInterestingObject
	^ self! !

!Object methodsFor: '*GT-Inspector' stamp: 'AndreiChis 12/13/2014 11:00'!
gtInspectorMetaIn: composite
	"This presentation offers the methods of the class of the current object"
	<gtInspectorPresentationOrder: 1000>
	<gtInspectorTag: #basic>
	^ composite custom: (GTSimpleMethodsBrowser new startOn: self) ! !

!Object methodsFor: '*GT-Inspector' stamp: 'AndreiChis 1/14/2015 14:22'!
gtInspectorPresentationsIn: composite inContext: aGTInspector
	"This is a utility method that collects all presentations for the current object.
	By default, it simply looks for the #gtInspectorPresentationOrder: pragma.
	The inspector can decice whether or not a presentation should be dispayed.
	Subclasses might want to override it for more special behavior."
	| pragmas |
	pragmas := Pragma 
				allNamed: #gtInspectorPresentationOrder: 
				from: self class 
				to: Object
				sortedUsing: [: x :y | (x argumentAt: 1) < (y argumentAt: 1)].
	pragmas := pragmas select: [ :aPragma | 
		aGTInspector shouldDisplayPresentationCreateBy: aPragma method ].
	pragmas do: [ :eachPragma | 
		(eachPragma selector findTokens: $:) size = 1 
			ifTrue: [ self perform: eachPragma selector with: composite ]. 
		(eachPragma selector findTokens: $:) size = 2
			ifTrue: [ self perform: eachPragma selector with: composite with: aGTInspector ] ]! !

!Object methodsFor: '*GT-Inspector' stamp: 'AndreiChis 12/13/2014 11:00'!
gtInspectorRawIn: composite
	"This is the most basic presentation showing the state of the object"
	<gtInspectorPresentationOrder: 10>
	<gtInspectorTag: #basic>
	^ composite custom: GTObjectVariablesBrowser new! !

!Object methodsFor: '*GT-Inspector' stamp: 'TudorGirba 12/26/2014 15:18'!
gtInspectorVariableValuePairs
	"This is a helper method that returns a collection of 
		variable_name -> value
	for the current object.
	Subclasses can override it to specialize what appears in the variables presentation"
	| bindings |
	bindings := self class allInstVarNames asSet
					collect: [ :iv | iv -> (self instVarNamed: iv) ].
	1 to: (self basicSize min: 42) do: [ :index | 
		bindings add: (index asTwoCharacterString -> (self basicAt: index)) ].
	^ bindings! !

!Object methodsFor: '*GT-Inspector-Examples' stamp: 'AndreiChis 10/30/2014 14:24'!
isGTExample
	
	^ false! !

!Collection methodsFor: '*GT-Inspector' stamp: 'TudorGirba 6/28/2014 22:19'!
gtInspectorInterestingObject
	^ self size = 1
		ifTrue: [ self anyOne ]
		ifFalse: [ self ]! !
GTInspector initialize!
